/* The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Eric Hodel's <drbrain@segment7.net> code.
 * 
 * The Initial Developer of the Original Code is Eric Hodel.
 * Portions created by Eric Hodel are Copyright (C) 2001 Eric
 * Hodel. All Rights Reserved.
 * 
 * Other Contributors:
 *      Christopher Hoess <choess@force.stwing.upenn.edu>
 *      Tim Taylor <tim@tool-man.org>
 */

function LanguageDictionary() {
  this.dictionary = null;

  this.lookupLanguageName = 
  function(languageCode) {
    if (this.getDictionary()[languageCode])
      return this.getDictionary()[languageCode];
    else
      return "";
  }

  this.getDictionary = function() {
    if (this.dictionary == null)
      this.dictionary = createDictionary();

    return this.dictionary;
  }

  function createDictionary() {
    ddump("createDictionary begin");
    var start = new Date();

    var dictionary = new Array();

    var e = getLanguageNames().getSimpleEnumeration();
    while (e.hasMoreElements()) {
      var property = e.getNext();
      property = property.QueryInterface(
          Components.interfaces.nsIPropertyElement);
      dictionary[property.getKey()] = property.getValue();
    }

    ddump("createDictionary end");
    var finish = new Date();
    ddump("createDictionary end");
    ddump("createDictionary: completed in "
        + (finish.getTime() - start.getTime()) + " ms");

    return dictionary;
  }

  function getLanguageNames() {
    return srGetStrBundle(
        "chrome://global/locale/languageNames.properties");
  }
}
const languageDictionary = new LanguageDictionary();


function LinkHandler() {
  this.items = new Array();
  this.defaultItem = null;

  this.registerItemForType = function(item) {
    this.items[item.type] = item;
  }

  this.registerDefaultItem = function(item) {
    this.defaultItem = item;
  }

  this.handleLinks = function(nodeList) {
    for (var i = 0; i < nodeList.length; i++) {
      if (nodeList.item(i).rel || nodeList.item(i).rev) 
        linkHandler.handle(nodeList.item(i));
    }
    ddump("LinkHandler::handleLinks: handled " + nodeList.length + " links");
  }

  this.handle = function(linkElement) {
    convertRevMade(linkElement);

    // ddump("LinkHandler::handle" + " rel=" + linkElement.rel + " title=" + linkElement.title + " media=" + linkElement.media + " hreflang=" + linkElement.hreflang);

    if (isExcludedLink(linkElement)) return;

    var relAttributes = linkElement.rel.split(" ");
    for (var i = 0; i < relAttributes.length; i++) {
      var type = getLinkType(relAttributes[i]);
      this.getLinkToolbarItemForType(type).addLink(linkElement);
    }
  }

  this.getLinkToolbarItemForType = function(type) {
    if (this.items[type] != null)
      return this.items[type];

    return this.defaultItem;
  }

  this.clear = function() {
    for (var type in this.items)
      this.items[type].clear();
    this.defaultItem.clear();
  }

  function convertRevMade(linkElement) {
    if (!linkElement.rel && linkElement.rev) {
      if (linkElement.rev.toLowerCase() == "made")
        linkElement.rel = linkElement.rev;
    }
  }

  function isExcludedLink(linkElement) {
    if (!linkElement.rel) return true;
    if (isStylesheetLink(linkElement)) return true;

    return false;
  }

  function isStylesheetLink(linkElement) {
    if (linkElement.rel.toLowerCase().search("stylesheet") != -1)
      return true;
    else
      return false;
  }

  function getLinkType(relAttribute) {
    switch (relAttribute.toLowerCase()) {
      case "home":
      case "start":
      case "top":
      case "origin":
        return "top";

      case "up":
      case "parent":
        return "up";

      case "begin":
      case "first":
        return "first";

      case "next":
      case "child":
        return "next";

      case "prev":
      case "previous":
        return "prev";

      case "end":
      case "last":
        return "last";

      case "author":
      case "made":
        return "author";

      case "contents":
      case "toc":
        return "toc";

      default:
        return relAttribute.toLowerCase();
    }
  }
}
const linkHandler = new LinkHandler();

function LinkToolbarItem (linkType) {
  this.type = linkType;

  this.element = function() {
    return document.getElementById("link-" + this.type);
  }

  this.clear = function() {
    this.element().setAttribute("disabled", "true");
    this.element().removeAttribute("href");
  }

  this.addLink = function(linkElement) {
    if (this.element().hasAttribute("href")) return;

    this.setItem(linkElement);
  }

  this.setItem = function(linkElement) {
    this.element().setAttribute("href", linkElement.href);
    this.element().removeAttribute("disabled");
  }

  createTooltip = function(linkElement) {
    var title = "";
    // XXX: don't prepend language name if a title is provided
    if (linkElement.hreflang)
      title += languageDictionary.lookupLanguageName(linkElement.hreflang)
            + " - ";
    if (linkElement.title)
      title += linkElement.title;
    else
      title += getNodeAsText(linkElement);

    // XXX: don't append media if title is provided
    if (linkElement.media)
      title += " - " + linkElement.media;

    return title;
  }

  getNodeAsText = function(node) {
    return condenseWhitespace(getNodeAsTextRecursive(node));
  }

  getNodeAsTextRecursive = function(node) {
    var text = "";
    node.normalize();
    if (node.hasChildNodes()) {
      for (var i = 0; i < node.childNodes.length; i++) {
        if (node.childNodes.item(i).nodeType == Node.TEXT_NODE)
          text += node.childNodes.item(i).nodeValue;
        else if (node.childNodes.item(i).nodeType == Node.ELEMENT_NODE)
          text += getNodeAsTextRecursive(node.childNodes.item(i));
      }
    }

    return text;
  }

  condenseWhitespace = function(text) {
    return text.replace(/\W*$/, "").replace(/^\W*/, "").replace(/\W+/g, " ");
  }
}

function LinkToolbarButton (linkType) {
  this.constructor(linkType);

  // XXX: copy-paste reuse from LinkToolbarItem::clear
  this.clear = function() {
    this.element().setAttribute("disabled", "true");
    this.element().removeAttribute("href");

    this.element().removeAttribute("tooltiptext");
  }

  // XXX: copy-paste reuse from LinkToolbarItem::setItem
  this.setItem = function(linkElement) {
    this.element().setAttribute("href", linkElement.href);
    this.element().removeAttribute("disabled");

    // XXX: don't want to fallback to using REL value like we
    //    do for menuitems.  Need to break out createTooltip
    //    into seperate methods
    this.element().setAttribute("tooltiptext", 
        createTooltip(linkElement));
  }
}
LinkToolbarButton.prototype = new LinkToolbarItem;


function LinkToolbarMenuItem (linkType) {
  this.constructor(linkType);

  // XXX: copy-paste-reuse from LinkToolbarItem::addLink
  this.addLink = function(linkElement) {
    if (this.element().hasAttribute("href")) return;

    this.setItem(linkElement);
    enableMoreMenu(); // XXX: kludge
  }
}
LinkToolbarMenuItem.prototype = new LinkToolbarItem;


function LinkToolbarMenu (linkType) {
  this.constructor(linkType);

  this.knownLinks = new Array();

  this.menupopup = function() {
    return document.getElementById("link-" + this.type + "-menupopup");
  }

  this.clear = function() {
    /*
     * XXX: use JS equivalent of Java's super.clear() idiom instead of
     *    copy-paste reuse from LinkToolbarItem::clear
     */
    this.element().setAttribute("disabled", "true");
    this.element().removeAttribute("href");
    if (this.menupopup())
      this.element().removeChild(this.menupopup());
    this.knownLinks = new Array();
  }

  this.addLink = function(linkElement) {
    if (this.isAlreadyAdded(linkElement)) return;

    this.knownLinks[linkElement.href] = true;
    this.addMenuitem(linkElement);
    enableMoreMenu(); // XXX: kludge
  }

  this.isAlreadyAdded = function(linkElement) {
    if (!this.knownLinks[linkElement.href]) return false;

    if (!matchMedia(this.knownLinks[linkElement.href], linkElement))
      return false;
    if (!matchHrefLang(this.knownLinks[linkElement.href], linkElement))
      return false;

    return true;
  }
  
  function matchMedia(link1, link2) {
    if (!link1.media && !link2.media) return true;
    if (!link1.media || !link2.media) return false;

    return link1.media == link2.media;
  }

  function matchHrefLang(link1, link2) {
    if (!link1.hreflang && !link2.hreflang) return true;
    if (!link1.hreflang || !link2.hreflang) return false;

    return link1.hreflang == link2.hreflang;
  }

  this.addMenuitem = function(linkElement) {
    var parentMenuItem = document.getElementById("link-" + this.type);
    parentMenuItem.removeAttribute("disabled");

    if (!this.menupopup()) {
      var popup = document.createElement("menupopup");
      popup.setAttribute("id", "link-" + this.type + "-menupopup");
      parentMenuItem.appendChild(popup);
    }

    menuitem = document.createElement("menuitem");
    menuitem.setAttribute("label", this.createLabel(
        linkElement, createTooltip(linkElement)));
    menuitem.setAttribute("href", linkElement.href);
	menuitem.setAttribute("class", "menuitem-iconic bookmark-item");
	menuitem.setAttribute("type", 
			"rdf:http://www.w3.org/1999/02/22-rdf-syntax-ns#type");

    this.menupopup().appendChild(menuitem);
    this.menupopup().parentNode.removeAttribute("disabled");
  }

  this.createLabel = function(linkElement, title) {
    if (title == "")
      return linkElement.rel;
    if (this.type == "misc")   // XXX: replace with polymorphism
      return linkElement.rel + ": " + title;

    return title;
  }

}
LinkToolbarMenu.prototype = new LinkToolbarItem;

function Init() {
  /* XXX: should generated this automatically from the XUL */
  linkHandler.registerItemForType(new LinkToolbarButton("top"));
  linkHandler.registerItemForType(new LinkToolbarButton("up"));
  linkHandler.registerItemForType(new LinkToolbarButton("first"));
  linkHandler.registerItemForType(new LinkToolbarButton("prev"));
  linkHandler.registerItemForType(new LinkToolbarButton("next"));
  linkHandler.registerItemForType(new LinkToolbarButton("last"));
  linkHandler.registerItemForType(new LinkToolbarMenuItem("help"));
  linkHandler.registerItemForType(new LinkToolbarMenuItem("search"));
  linkHandler.registerItemForType(new LinkToolbarMenuItem("author"));
  linkHandler.registerItemForType(new LinkToolbarMenuItem("copyright"));
  linkHandler.registerItemForType(new LinkToolbarMenuItem("toc"));
  linkHandler.registerItemForType(new LinkToolbarMenu("chapter"));
  linkHandler.registerItemForType(new LinkToolbarMenu("section"));
  linkHandler.registerItemForType(new LinkToolbarMenu("subsection"));
  linkHandler.registerItemForType(new LinkToolbarMenu("appendix"));
  linkHandler.registerItemForType(new LinkToolbarMenuItem("glossary"));
  linkHandler.registerItemForType(new LinkToolbarMenuItem("index"));
  linkHandler.registerItemForType(new LinkToolbarMenu("bookmark"));
  linkHandler.registerItemForType(new LinkToolbarMenu("alternate"));
  linkHandler.registerDefaultItem(new LinkToolbarMenu("misc"));
}

/* 
 * refresh is called on every page load 
 *
 * FIXME: nothing shows up until after the entire page has finished loading
 * FIXME: browser becomes unresponsive during this function...for as
 *    long as 2 or more seconds on heavily linked documents
 * XXX: optimization: don't refresh when page reloaded if the page hasn't
 *    been modified
 * XXX: optimization: don't refresh when toolbar is compacted
 */
function refresh() {
  ddump("refresh begin");
  var start = new Date();

  clear();

  linkHandler.handleLinks(getLinkElements());
  linkHandler.handleLinks(getAnchorElements());
  
  var finish = new Date();
  ddump("refresh end");
  ddump("refresh: completed in " + (finish.getTime() - start.getTime()) + " ms");
}

function getLinkElements() {
  return window._content.document.getElementsByTagName("LINK");
}

function getAnchorElements() {
  return window._content.document.getElementsByTagName("A");
}

function clear() {
  linkHandler.clear();
  disableMoreMenu(); // XXX: kludge
}

/* Process a click */
function clicked(event) {
  if (event.target.nodeName == "menubutton") return;
  if (!event.target.getAttribute("href")) return;
  
  ddump("clicked loadURI(" + event.target.getAttribute("href") + ")");
  loadURI(event.target.getAttribute("href"));
}

// XXX: two kludges until More menu disabling/enabling is done
//      properly with Observers
function enableMoreMenu() {
  document.getElementById("link-more").removeAttribute("disabled");
}
function disableMoreMenu() {
  document.getElementById("link-more").setAttribute("disabled", "true");
}

function ddump(msg) {
  dump("LinkToolbar::" + msg + "\n");
}

